# Example: look at track resolution using single particles
# S.Chekanov (ANL)

from org.lcsim.lcio import LCIOReader
from hep.io.sio import SIOReader
from hep.lcio.implementation.sio import SIOLCReader
from hep.lcio.implementation.io import LCFactory
from hep.lcio.event import * 
from hep.lcio.io import *
from jhplot import *  # import graphics
from hephysics.particle import LParticle
from java.lang import System
from java.awt import Font,Color
from jhplot.io import HBook
import sys,math

# pT bins (min, max)
ptbins=[[0,10],[10,20],[20,30],[30,40],[40,50],[50,60],[60,70],[70,80],[80,90],[90,100],[100,110],[110,120],[120,130],[130,140],[140,150]]

# fit ranges 
xmin=-0.01
xmax=0.01

# make histograms for ratio and pT bins
h_re=[] # histogram with ratio 
h_pt=[] # histogram with pT 
print "Nr of bins=",len(ptbins)
for j in range(len(ptbins)):
        pbin=ptbins[j]
        lab=str(pbin[0])+"-"+str(pbin[1])
        xmin=-300/(1+0.4*j)  
        xmax= 300/(1+0.4*j)  
        print xmin, xmax
        h_re.append(H1D("ratio: "+lab,150,xmin,xmax))
        h_pt.append(H1D("bin: "+lab,150,pbin[0],pbin[1]))

# make list of files..
import glob
files=glob.glob("data/*.slcio")
factory = LCFactory.getInstance()

nEvent=0

for f in files:
  print "Open file=",f
  reader = factory.createLCReader()
  reader.open(f) 
  while(1):
    evt=reader.readNextEvent()
    if (evt == None): break
    nEvent=nEvent+1
    # print " file event: ",evt.getEventNumber(), " run=",evt.getRunNumber()
    if (nEvent%100==0): print "# Event: ",nEvent
    col = evt.getCollection("MCParticle")
    nMc=col.getNumberOfElements()
    particles=[]
    pp=0
    nn=0
    for i in range(nMc): # loop over all particles 
      par=col.getElementAt(i)
      if(par.getGeneratorStatus() == 1 and par.getCharge() !=0): 
              vertex = par.getVertex();
              pdg=par.getPDG()
              momentum = par.getMomentum() 
              ee=par.getEnergy()
              mass=par.getMass()
              px=momentum[0]
              py=momentum[1]
              pz=momentum[2]
              charge=par.getCharge()
              p=LParticle(px,py,pz,ee,mass) 
              p.setCharge(charge)
              theta=math.atan2(p.perp(),pz)
              theta_rad=theta*57.2958
              p.setCharge(charge)
              if (theta_rad>10 and theta_rad<40): particles.append(p)
              if (theta_rad>140 and theta_rad<170): particles.append(p)
              nn=nn+1

    pfo=[]
    col = evt.getCollection("Tracks")
    tracks = col.getNumberOfElements()
    for i in range(tracks):
       # http://www.lcsim.org/sites/lcsim/apidocs/org/lcsim/lcio/SIOReconstructedParticle.html
       # http://www.lcsim.org/sites/lcsim/apidocs/org/lcsim/event/ReconstructedParticle.html
       pa=col.getElementAt(i)
       d0=pa.getD0()
       z0=pa.getZ0()
       for i2 in range(len(particles)):
            p2=particles[i2]
            t_pt=p2.perp()
            for j in range(len(ptbins)):
                    pbin=ptbins[j]
                    if(t_pt>pbin[0] and t_pt<pbin[1]):
                             h_re[j].fill(d0*1000)
                             h_pt[j].fill(t_pt)  


    del col
    del evt 
  reader.close() # close the file
  del reader
  System.gc() # if you low on memory, manual JVM cleanup for every 5th file 

pres=P1D("Endcap")
# fit histograms with a Gaussian
for i in range(len(ptbins)):
    pbin=ptbins[i]
    lab=str(pbin[0])+"-"+str(pbin[1])
    h=h_re[i]
    p=h_pt[i]
    f=HFitter("chi2")   # build fitter 
    f.setFunc("g")      # gaussian + constant  
    func=f.getFunc()
    xmin=-300/(1+0.4*i)  
    xmax= 300/(1+0.4*i)   
    #print func.parameterNames()
    f.setPar('mean',h.mean()); f.setPar('amplitude',h.maxBinHeight()), f.setPar('sigma',h.rms()) 
    f.setRange(xmin,xmax) # fit range 
    f.fit(h)
    fresult=f.getFittedFunc() # return final function 
    res=f.getResult()
    fPars  = res.fittedParameters()
    fPars     = res.fittedParameters()
    fParErrs  = res.errors()
    fParNames = res.fittedParameterNames()
    print "Fit results:"
    for i in range(fresult.numberOfParameters()):
       print "par=",i,"=", fParNames[i]+" : "+str(fPars[i])+" +- "+str(fParErrs[i])
    p2=P1D(h)   # convert histogram to P1D array
    # c1.draw(p2)
    # make F1D function from IFunction in the same range as histogram
    f2 = F1D("Gauss fit",fresult,xmin,xmax)
    f2.setColor(Color.blue)
    f2.setPenWidth(2);
    # c1.draw(f2) # draw fit line  
    q1="mean=%4.3f &plusmn; %4.3f GeV"%(fPars[1],fParErrs[1])
    print "pT=",p.mean()," GeV ", q1

    c1=HPlot("PT="+lab,600,400)
    c1.visible()
    c1.setMarginLeft(80)
    c1.setNameX("d0 [&mu;m]")
    c1.setNameY("Events:"+lab)
    c1.setAutoRange()
    c1.setRangeX(xmin,xmax)
    print "Draw in range",xmin,xmax
    c1.draw(p2)
    c1.draw(f2)
    # x, y, err_x, err_x, err_y, err_y
    pres.add(p.mean(),fPars[2],p.rms(),p.rms(),fParErrs[2],fParErrs[2])
    raw_input("Press Enter to continue...")

# final plot
c1=HPlot("resolution",600,400)
c1.visible()
c1.setNameX("pT [GeV]")
c1.setNameY("&delta; (d0) [&mu;m]")
c1.setAutoRange()
c1.setMarginLeft(100)
c1.setRangeX(0,150)
c1.setRangeY(0,20)

c1.setLegend(False)
l1=HLabel("Endcap", 0.6, 0.8, "NDC")
l1.setFont(Font("Helvetica", Font.PLAIN, 20))
c1.add(l1)

c1.draw(pres)
# print pres.toString()

name="output"
if len(sys.argv[0])>0: name=sys.argv[0].replace(".py","")
f=HBook(name+".jdat","w"); print name+".jdat created"
f.write(pres)
f.close()
c1.export(name+".svg");    print name+".svg created"

